import winston from 'winston';
import { check, MessageType, MiraiApiHttpSetting, Contact, MiraiInstance, Api } from 'mirai-ts';
import commander from 'commander';
import mongoose from 'mongoose';
import events from 'events';
import { Server } from 'net';
import Koa from 'koa';
import { Webhooks } from '@octokit/webhooks';
import nodeSchdule from 'node-schedule';

interface Logger extends winston.Logger {
    success: winston.LeveledLogMethod;
}
declare function createLogger(label?: string): Logger;

interface WebhookConfig {
    enable?: boolean;
    port?: number;
    path?: string;
    secret?: string;
    callback?: (webhook: any) => {};
}
declare class Webhook {
    ctx: Bot;
    config: WebhookConfig;
    emitter: events.EventEmitter;
    githubHandler: Webhooks<any>;
    middleware: (request: any, response: any, next?: Function | undefined) => Promise<any>;
    constructor(ctx: Bot);
    start(config?: WebhookConfig): Server | undefined;
    parse(ctx: Koa.ParameterizedContext<Koa.DefaultState, Koa.DefaultContext>): void;
    on(type: string, callback: Function): void;
}

interface Target {
    friend?: number[];
    group?: number[];
}
declare type BaseListenType = 'all' | 'master' | 'admin' | 'friend' | 'group';
declare type Listen = Target | (BaseListenType | number)[];

type config_Target = Target;
type config_BaseListenType = BaseListenType;
type config_Listen = Listen;
declare namespace config {
  export {
    config_Target as Target,
    config_BaseListenType as BaseListenType,
    config_Listen as Listen,
  };
}

declare type ReplyContent = string | Partial<MessageType.SingleMessage>[];
interface BaseAnswerOptions extends check.Match {
    listen?: string | Listen;
    unlisten?: Listen;
    cron?: nodeSchdule.RecurrenceRule;
    target?: Target;
    api?: string;
    reply: ReplyContent;
    at?: boolean;
    quote?: boolean;
    else?: ReplyContent;
    help?: string;
}
declare type AnswerOptions = BaseAnswerOptions[];

interface ForwardItem {
    listen: Listen;
    target: Target;
}
declare type ForwardOptions = ForwardItem[];
declare function export_default(ctx: Bot, options: ForwardOptions): void;

interface BotConfig {
    name: string;
    autoloadPlugins: boolean;
    pluginDir: string;
    plugins: {
        default?: string[];
        official?: string[];
        community?: string[];
        custom?: string[];
    };
    master: number[];
    admin: number[];
    devGroup: number;
    answer?: AnswerOptions;
    forward?: ForwardOptions;
    [propName: string]: any;
}
declare type BotUserConfig = Partial<BotConfig>;

interface dbConfig {
    enable: boolean;
    uri?: string;
    analytics?: Boolean;
}
interface reportConfig {
    enable: boolean;
    target?: Target;
}
interface ElConfig<T = BotConfig> {
    qq: number;
    setting: MiraiApiHttpSetting | string;
    mirai: {
        folder: string;
    };
    db: dbConfig;
    bot: T;
    webhook: WebhookConfig;
    report: reportConfig;
    pkg?: any;
    base: string;
    path: {
        image: string;
        voice: string;
    };
}
declare type ElUserConfig = Partial<ElConfig<BotUserConfig>>;
declare function resolveElConfig(userConfig: ElUserConfig): ElConfig<BotConfig>;

declare class Sender {
    ctx: Bot;
    constructor(ctx: Bot);
    sendFriendMessageByArray(messageChain: string | MessageType.MessageChain, array: number[], messageList: number[]): Promise<void[]>;
    sendMessageByConfig(messageChain: string | MessageType.MessageChain, target: Target): Promise<number[]>;
}

declare class User {
    ctx: Bot;
    constructor(ctx: Bot);
    isMaster(qq: number): boolean;
    isAdmin(qq: number): boolean;
    isAllowed(qq?: number, reply?: boolean, content?: string): boolean;
}

declare class Status {
    ctx: Bot;
    constructor(ctx: Bot);
    isListening(sender: Contact.User, listen: Listen): boolean;
    getListenStatusByConfig(sender: Contact.User, config: any): boolean;
}

declare type PluginInstallFunction = (ctx: Bot, ...options: any[]) => any;
interface PluginInfo {
    name?: string;
    version?: string;
    description?: string;
    pkg?: object;
}
declare type Plugin = ({
    install: PluginInstallFunction;
} & PluginInfo);
declare type PluginType = 'default' | 'official' | 'community' | 'custom';
declare const PluginTypeMap: Record<PluginType, string>;
declare class Plugins {
    ctx: Bot;
    default: Set<PluginInfo>;
    official: Set<PluginInfo>;
    community: Set<PluginInfo>;
    custom: Set<PluginInfo>;
    constructor(ctx: Bot);
    isOfficial(name: string): boolean;
    isCommunity(name: string): boolean;
    getPluginFullName(name: string, type: PluginType): string;
    load(type: PluginType): void;
    isBasedOnDb(pkg: any): boolean;
    add(name: string, plugin: Plugin, options?: any): void;
    list(type: PluginType): string;
}

declare class Command {
    ctx: Bot;
    name: string;
    desc: string;
    callback: (options: string[]) => void;
    children: Map<string, Command>;
    constructor(ctx: Bot);
    command(name: string): this;
    description(desc: string): this;
    action(callback: (options: string[]) => any): this;
    parse(text: string): void;
    listen(): void;
}

declare function createBot(el: ElUserConfig): Bot;
declare class Bot {
    el: ElConfig;
    mirai: MiraiInstance;
    active: boolean;
    db?: mongoose.Connection;
    status: Status;
    user: User;
    sender: Sender;
    plugins: Plugins;
    installedPlugins: Set<unknown>;
    cli: commander.Command;
    _command: Command;
    logger: Logger;
    webhook?: Webhook;
    isDev: boolean;
    rootDir: string;
    tmpDir: string;
    isTS: boolean;
    constructor(el: ElUserConfig);
    reply(msgChain: string | MessageType.MessageChain, quote?: boolean): false | Promise<void>;
    link(): Promise<Api.Response.BaseResponse | undefined>;
    start(): Promise<void>;
    use(plugin: Plugin | PluginInstallFunction, ...options: any[]): this;
    plugin(name: string, plugin: Plugin | PluginInstallFunction, ...options: any[]): void;
    command(name: string): Command;
}

export { AnswerOptions as A, BotConfig as B, ElUserConfig as E, ForwardOptions as F, Listen as L, PluginInstallFunction as P, Target as T, BotUserConfig as a, Bot as b, createBot as c, createLogger as d, config as e, dbConfig as f, ElConfig as g, resolveElConfig as h, export_default as i, PluginInfo as j, Plugin as k, PluginType as l, PluginTypeMap as m, Plugins as n, reportConfig as r };
